from scipy.io import loadmat
import pandas as pd
import altair as alt
import numpy as np

def flatten_signal(raw):
    """
    Takes the raw EMG signal array, flattens it, and removes empty channels with no data.
    
    Parameters
    ----------
    raw: numpy.ndarray
        Raw EMG signal array.
        
    Returns
    -------
    numpy.ndarray
        Flattened EMG signal array, with empty channels removed.
    """
    # Flatten input array
    raw_flattened = raw.flatten()
    # Remove empty channels and then removes dimension of size 1
    raw_flattened = np.array([channel for channel in raw_flattened if 0 not in channel.shape]).squeeze()

    return raw_flattened

def extend_input_by_R(x, R):
    """
    Takes a one-dimensional array and extends it using past observations.

    Parameters
    ----------
        x: numpy.ndarray
            1D array to be extended.
        R: int
            How far to extend x.
    Returns
    -------
        numpy.ndarray
            len(x) by R+1 extended array.

    Examples
    --------
        >>> R = 5
        >>> x = np.array([1, 2, 3])
        >>> extend_input_by_R(x, R)
        array([[1., 0., 0., 0., 0., 0.],
               [2., 1., 0., 0., 0., 0.],
               [3., 2., 1., 0., 0., 0.]])
    """

    # Create array with R+1 rows and length of x + R columns
    extended_x = np.zeros((R + 1, len(x) + R))

    # Create array where each row is a delayed version of the previous row
    for i in range(R + 1):
        extended_x[i][i : i + len(x)] = x

    # Transpose array and cut off extra R rows
    extended_x = extended_x.T[0 : len(x)]

    return extended_x

def extend_input_all_channels(x_mat, R):
    """
    Takes an array with dimensions m by k,
    where m represents channels and k represents observations,
    and "extends" it to return an array of shape m by k by R+1.
    
    Parameters
    ----------
        x_mat: numpy.ndarray
            2D array to be extended.
        R: int
            How far to extend x.
        
    Returns
    -------
        numpy.ndarray
            m by k by R+1 extended array.
        
    Example:
        >>> R = 5
        >>> x_mat = np.array([[1, 2, 3], [4, 5, 6]])
        >>> extend_input_all_channels(x_mat, R)
        array([[[1., 0., 0., 0., 0., 0.],
                [2., 1., 0., 0., 0., 0.],
                [3., 2., 1., 0., 0., 0.]],

               [[4., 0., 0., 0., 0., 0.],
                [5., 4., 0., 0., 0., 0.],
                [6., 5., 4., 0., 0., 0.]]])

    """
    extended_x_mat = np.zeros([x_mat.shape[0], x_mat.shape[1], R + 1])
    for i, channel in enumerate(x_mat):
        
        # Extend channel
        extended_channel = extend_input_by_R(channel, R)

        # Add extended channel to the overall matrix of extended channels
        extended_x_mat[i] = extended_channel
        
    return extended_x_mat



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['emgdecompy']

package_data = \
{'': ['*']}

install_requires = \
['altair-data-server>=0.4.1,<0.5.0',
 'altair>=4.2.0,<5.0.0',
 'numpy>=1.22.3,<2.0.0',
 'pandas>=1.4.2,<2.0.0',
 'scipy>=1.8.0,<2.0.0']

setup_kwargs = {
    'name': 'emgdecompy',
    'version': '0.1.0',
    'description': 'A package for decomposing raw EMG signals into individual motor unit activity.',
    'long_description': '# emgdecompy\n[![ci-cd](https://github.com/UBC-SPL-MDS/emg-decomPy/actions/workflows/ci-cd.yml/badge.svg)](https://github.com/UBC-SPL-MDS/emg-decomPy/actions/workflows/ci-cd.yml)\n\nA package for decomposing raw EMG signals into individual motor unit activity.\n\n## Proposal\n\nOur project proposal can be found [here](https://github.com/UBC-SPL-MDS/emg-decomPy/blob/main/docs/proposal/proposal.pdf).\n\nTo generate the proposal locally, run the following command from the root directory:\n\n```Rscript -e "rmarkdown::render(\'docs/proposal/proposal.Rmd\')"```\n\n## Installation\n\n```bash\npip install emgdecompy\n```\n\n## Usage\n\n- TODO\n\n## Contributing\n\nInterested in contributing? Check out the contributing guidelines. Please note that this project is released with a Code of Conduct. By contributing to this project, you agree to abide by its terms.\n\n## License\n\n`emgdecompy` was created by Daniel King. It is licensed under the terms of the MIT license.\n\n## Credits\n\n`emgdecompy` was created with [`cookiecutter`](https://cookiecutter.readthedocs.io/en/latest/) and the `py-pkgs-cookiecutter` [template](https://github.com/py-pkgs/py-pkgs-cookiecutter).\n',
    'author': 'Daniel King',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<3.11',
}


setup(**setup_kwargs)
